from abc import ABC
from vlcishared.tokens.arcgis_token_provider_config import ArcGISTokenProviderConfig
from vlcishared.tokens.token_provider import TokenProvider


class ArcGISTokenProvider(TokenProvider, ABC):
    """
    Clase base abstracta para proveedores de tokens de ArcGIS.
    Define la estructura y delega en las subclases la generación
    del payload concreto.
    """

    def __init__(self, config: ArcGISTokenProviderConfig):
        super().__init__(config.url_token_provider)
        self.username = config.username
        self.password = config.password
        self.format = config.format
        self.client = config.client
        self.expiration = config.expiration

    def obtener_token(self):
        """
        Realiza la solicitud del token al servidor ArcGIS usando las credenciales
        y parámetros configurados. Asigna el token recibido a self.token.

        Lanza:
            ValueError: Si la respuesta no contiene un token válido.

        Devuelve:
            Token obtenido
        """
        payload = self.get_payload()
        respuesta = self._solicitar_nuevo_token(data=payload)
        token = respuesta.get("token")
        if token is None:
            raise ValueError("No se ha podido obtener el token.")
        self.token = token
        
        return token

    def get_payload(self):
        """
        Generación del payload específico de la solicitud de token.
        """
        return {
            "username": self.username,
            "password": self.password,
            "f": self.format,
            "client": self.client,
            "expiration": self.expiration,
        }


from vlcishared.http.http import ClienteHTTP


class TokenProvider:
    """
    Clase abstracta encargada de encapsular la lógica genérica para obtener un token vía HTTP POST.

    Parámetros:
        url: URL a la que se enviará la petición para obtener el token.
    """

    def __init__(self, url_token_provider):
        self.url_token_provider = url_token_provider
        self.cliente_http = ClienteHTTP()
        self.token = None

    def _solicitar_nuevo_token(self, *, data=None, files=None):
        """
        Realiza una solicitud POST a la URL configurada con los datos proporcionados.

        Parámetros:
            data (dict, opcional): Datos del formulario.
            files (dict, opcional): Archivos para enviar.

        Retorna:
            dict: La respuesta del servidor como objeto JSON.
        """
        response = self.cliente_http.post(url=self.url_token_provider, data=data, files=files)
        return response

    def obtener_token(self):
        """
        Método abstracto que deben implementar las subclases para establecer self.token.

        Lanza:
            NotImplementedError: Si no se implementa en una subclase.
        """
        raise NotImplementedError("Este método debe ser implementado por las subclases")



from email.mime.application import MIMEApplication
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import os

from vlcishared.mail.mail_server import EmailServer


class EmailSender:

    def __init__(self, email_server: EmailServer, sender, sender_name, subject, to, cc=None, bcc=None):
        """Inicializa el EmailSender con la configuración del correo."""
        self.email_server = email_server
        self.sender = sender
        self.sender_name = sender_name
        self.subject = subject
        self.to = to
        self.cc = cc if cc else ""
        self.bcc = bcc if bcc else ""

        self.msg = MIMEMultipart()
        self.msg["From"] = f"{self.sender_name} <{self.sender}>"
        self.msg["To"] = self.to
        self.msg["Cc"] = self.cc
        self.msg["Subject"] = self.subject
        self.body = ""

    def append_line(self, message):
        """Añade una línea al cuerpo del correo en HTML."""
        self.body += message + "<br>"

    def add_attachment(self, file_path, subtype="octet-stream"):
        """Adjunta un archivo al correo."""
        file_name = file_path.split("/")[-1]
        with open(file_path, "rb") as file:
            attachment = MIMEApplication(file.read(), _subtype=subtype)
            attachment.add_header("content-disposition", "attachment", filename=file_name)
            self.msg.attach(attachment)

    def add_inline_attachment(self, file_path, cid_name):
        """
        Adjunta una imagen para mostrarla inline en el correo HTML.
        `cid_name` será el Content-ID y lo usarás en tu HTML: <img src="cid:cid_name">
        """
        file_name = os.path.basename(file_path)
        with open(file_path, "rb") as file:
            img = MIMEImage(file.read())
            img.add_header("Content-ID", f"<{cid_name}>")
            img.add_header("Content-Disposition", "inline", filename=file_name)
            self.msg.attach(img)

    def send(self):
        """Envía el correo usando la conexión SMTP."""
        self.msg.attach(MIMEText(self.body, "html"))
        recipients = self.to.split(",") + self.cc.split(",") + self.bcc.split(",")
        recipients = [r.strip() for r in recipients if r.strip()]

        smtp_connection = self.email_server.get_smtp_connection()
        smtp_connection.sendmail(self.sender, recipients, self.msg.as_string())
        smtp_connection.quit()
